#!/usr/bin/perl
#
use Getopt::Long;
use DBI;


my($USAGE) = "Usage: $0: [-d] [--dbname=database] [--dbuser=username] [--dbpass=password] [--dbhost=hostname] [--dbschema=schemaname] [--batch] [--preparedbi] [--prepare] [--array] <accumtest>...\n";
my(%OPTIONS);
Getopt::Long::Configure("bundling", "no_ignore_case", "no_auto_abbrev", "no_getopt_compat", "require_order");
GetOptions(\%OPTIONS, 'debug|d+', 'dbname=s', 'dbuser=s', 'dbpass=s', 'dbhost=s', 'dbschema=s', 'batch', 'preparedbi', 'prepare', 'array') || die $USAGE;

my (@dsn);

push(@dsn,"dbname=$OPTIONS{'dbname'}") if ($OPTIONS{'dbname'});
push(@dsn,"host=$OPTIONS{'dbhost'}") if ($OPTIONS{'dbhost'});
push(@dsn,"port=$OPTIONS{'dbport'}") if ($OPTIONS{'dbport'});
push(@dsn,"options=$OPTIONS{'dboptions'}") if ($OPTIONS{'dboptions'});
push(@dsn,"tty=$OPTIONS{'dbtty'}") if ($OPTIONS{'dbtty'});
#push(@dsn,"username=$OPTIONS{'dbuser'}") if ($OPTIONS{'dbuser'});
#push(@dsn,"password=$OPTIONS{'dbpass'}") if ($OPTIONS{'dbpass'});

my $dbh = DBI->connect("dbi:Pg:".join(";",@dsn), $OPTIONS{'dbuser'}, $OPTIONS{'dbpass'}, { AutoCommit => 1, RaiseError => 0, PrintError => 0, Warn => 0 } ) || die $DBI::errstr;


if ($OPTIONS{'batch'})
{
  $dbh->begin_work;
}
if ($OPTIONS{'preparedbi'})
{
  my $tup = $dbh->prepare(q^UPDATE test SET accum = accum + ? WHERE val = ?;^);
  my $tin = $dbh->prepare(q^INSERT INTO test VALUES(?, ?);^);
  while (<>)
  {
    chomp;
    @A = split;

    if ($tup->execute($A[1], $A[0]) eq "0E0")
    {
      $tin->execute($A[0], $A[1]) || die;
    }
  }
}
elsif ($OPTIONS{'prepare'})
{
  die "Does not work yet--Postgres cannot return result of execute";
  dosql($dbh, q^PREPARE testupdate(BIGINT, INTEGER) AS UPDATE test SET accum = accum + $2 WHERE val = $1;^);
  dosql($dbh, q^PREPARE testinsert(BIGINT, INTEGER) AS INSERT INTO test VALUES($1, $2);^);
  while (<>)
  {
    chomp;
    @A = split;

    if (dosql($dbh, "EXECUTE UPDATE test SET accum = accum + $A[1] WHERE val = '$A[0]';",0) eq "0E0")
    {
      dosql($dbh, "INSERT INTO test VALUES ( $A[0], $A[1] );");
    }
  }
}
elsif ($OPTIONS{'array'})
{
  while (<>)
  {
    chomp;
    @A = split;

    if (dosql($dbh, "UPDATE testarray SET accum[2] = accum[2] + $A[1] WHERE val = '$A[0]';",0) eq "0E0")
    {
      dosql($dbh, "INSERT INTO testarray VALUES ( $A[0], '{1,$A[1],3,4}' );");
    }
  }
}
else
{
  while (<>)
  {
    chomp;
    @A = split;

    if (dosql($dbh, "UPDATE test SET accum = accum + $A[1] WHERE val = '$A[0]';",0) eq "0E0")
    {
      dosql($dbh, "INSERT INTO test VALUES ( $A[0], $A[1] );");
    }
  }
}
if ($OPTIONS{'batch'})
{
  $dbh->commit;
}
$dbh->disconnect() || die "Database error ".$dbh->errstr;



######################################################################
# Imperative SQL commands
#
sub dosql($$;$)
{
  my ($dbh,$sql,$flags) = @_;

  print STDERR "SQL>$sql\n" if ($OPTIONS{'debug'});
  my $rv;
  if (!($rv = $dbh->do($sql)))
  {
    print STDERR "SQL!<$rv><".$dbh->errstr."><".$dbh->err.">\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot execute >$sql<: ".$dbh->errstr."\n";
  }

  print STDERR "SQL<$rv\n" if ($OPTIONS{'debug'});

  $rv;
}



######################################################################
# Reactive SQL commands
#
sub querysql($$;$)
{
  my ($dbh,$sql,$flags) = @_;

  print STDERR "SQL>$sql\n" if ($OPTIONS{'debug'});
  my $sth;
  if (!($sth = $dbh->prepare($sql)))
  {
    print STDERR "SQL!<err><".$dbh->errstr."\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot prepare >$sql<: ".$dbh->errstr."\n";
  }

  my $rv;
  if (!($rv = $sth->execute))
  {
    print STDERR "SQL!<$rv><".$dbh->errstr."\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot execute >$sql<: ".$dbh->errstr."\n";
  }
  print STDERR "SQL<$rv\n" if ($OPTIONS{'debug'});
  $sth;
}
