#!/usr/bin/perl
#
#

use Getopt::Long;
use DBI;


my($USAGE) = "Usage: $0: [-d] [--dbname=database] [--dbuser=username] [--dbpass=password] [--dbhost=hostname] [--dbschema=schemaname] [--batch] [--preparedbi] \n";
my(%OPTIONS);
Getopt::Long::Configure("bundling", "no_ignore_case", "no_auto_abbrev", "no_getopt_compat", "require_order");
GetOptions(\%OPTIONS, 'debug|d+', 'dbname=s', 'dbuser=s', 'dbpass=s', 'dbhost=s', 'dbschema=s') || die $USAGE;

my (@dsn);

push(@dsn,"dbname=$OPTIONS{'dbname'}") if ($OPTIONS{'dbname'});
push(@dsn,"host=$OPTIONS{'dbhost'}") if ($OPTIONS{'dbhost'});
push(@dsn,"port=$OPTIONS{'dbport'}") if ($OPTIONS{'dbport'});
push(@dsn,"options=$OPTIONS{'dboptions'}") if ($OPTIONS{'dboptions'});
push(@dsn,"tty=$OPTIONS{'dbtty'}") if ($OPTIONS{'dbtty'});
#push(@dsn,"username=$OPTIONS{'dbuser'}") if ($OPTIONS{'dbuser'});
#push(@dsn,"password=$OPTIONS{'dbpass'}") if ($OPTIONS{'dbpass'});

my $dbh = DBI->connect("dbi:Pg:".join(";",@dsn), $OPTIONS{'dbuser'}, $OPTIONS{'dbpass'}, { AutoCommit => 1, RaiseError => 0, PrintError => 0, Warn => 0 } ) || die $DBI::errstr;


while (<>)
{
  chomp;
  @A = split;

  my $sth = querysql($dbh,"SELECT accum FROM test WHERE val = $A[0];");

  my $row;
  if ($sth && ($row = $sth->fetchrow_arrayref))
  {
#    print "$A[0] $row->[0]\n";
    $hit++;
    $tot++;
  }
  else
  {
#    print "$A[0] -\n";
    $tot++;
  }
}
$dbh->disconnect() || die "Database error ".$dbh->errstr;
print "$hit / $tot = ".($hit/$tot)."\n";


######################################################################
# Imperative SQL commands
#
sub dosql($$;$)
{
  my ($dbh,$sql,$flags) = @_;

  print STDERR "SQL>$sql\n" if ($OPTIONS{'debug'});
  my $rv;
  if (!($rv = $dbh->do($sql)))
  {
    print STDERR "SQL!<$rv><".$dbh->errstr."><".$dbh->err.">\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot execute >$sql<: ".$dbh->errstr."\n";
  }

  print STDERR "SQL<$rv\n" if ($OPTIONS{'debug'});

  $rv;
}



######################################################################
# Reactive SQL commands
#
sub querysql($$;$)
{
  my ($dbh,$sql,$flags) = @_;

  print STDERR "SQL>$sql\n" if ($OPTIONS{'debug'});
  my $sth;
  if (!($sth = $dbh->prepare($sql)))
  {
    print STDERR "SQL!<err><".$dbh->errstr."\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot prepare >$sql<: ".$dbh->errstr."\n";
  }

  my $rv;
  if (!($rv = $sth->execute))
  {
    print STDERR "SQL!<$rv><".$dbh->errstr."\n" if ($OPTIONS{'debug'});
    return undef if ($flags & 1);
    die "Cannot execute >$sql<: ".$dbh->errstr."\n";
  }
  print STDERR "SQL<$rv\n" if ($OPTIONS{'debug'});
  $sth;
}
